@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+﻿# -*- coding: utf-8 -*-
 """
 Created on Thu Jun 16 10:38:52 2011
 
@@ -6,118 +6,16 @@ Created on Thu Jun 16 10:38:52 2011
 """
 from utils import ArithmeticBase
 from errorvalue import ErrorValue
-from attributealias import AliasedAttributes
+from attributealias import AliasedAttributes,AliasedArrayAttributes
 
 import numpy as np
 import matplotlib
-# IPython has a wrapper for matplotlib.use to inhibit switching backends.
-# It accepts only one argument however, not two, so we have to work it around.
-try:
-    matplotlib.use('TkAgg',warn=False)
-except TypeError:
-    pass
 
-import matplotlib.backends
-if not matplotlib.backends.backend=='TkAgg':
-    raise RuntimeError('Cannot work with other matplotlib backend (currently %s) than TkAgg.'%matplotlib.backends.backend)
 import matplotlib.pyplot as plt
 from matplotlib.cbook import is_numlike
 import scipy.optimize
 import fitfunction
  
-class AliasedArrayAttributes(AliasedAttributes):
-    """This mixin class adds support for aliased array attributes. It inherits
-    from AliadedAttributes to perform attribute aliasing. All aliased attributes
-    need to be numpy ndarrays of the same shape. This is ensured by a validating
-    function _attr_validate().
-    
-    The attribute initialization mechanism of AliasedAttributes is used to
-    initialize not available attributes to zero if they are referenced. 
-    
-    The constructor accepts 'normalnames', a keyword argument. It should be a
-    list defining the names of the aliased attributes. Their unaliased
-    counterparts will be constructed by prepending an underscore to their names.
-    The keyword argument 'keytrans' will be constructed appropriately for
-    AliasedAttributes. If 'keytrans' was supplied to the constructor, it will be
-    updated accordingly.
-    
-    Slicing and array-like indexing is implemented (only getting, but not
-    setting or deleting slices).
-    """
-    def __init__(self, **kwargs):
-        if 'normalnames' not in kwargs.keys():
-            kwargs['normalnames'] = []
-        self._normalnames = kwargs['normalnames']
-        del kwargs['normalnames']
-        self._protnames = ['_%s'%a for a in self._normalnames]
-        if 'keytrans' not in kwargs.keys():
-            kwargs['keytrans'] = {}
-        
-        #update 'keytrans'
-        kwargs['keytrans'].update(dict(zip(self._normalnames, self._protnames)))
-        AliasedAttributes.__init__(self, **kwargs)
-        self._shape = None
-    def _init_attribute(self,name):
-        #this is called by AliasedAttributes.__getattr__ if 'name' is not found.
-        
-        #if '_shape' is None or an empty list, we cannot do anything, raise an
-        # exception
-        shape=object.__getattribute__(self, '_shape')
-        if not shape: 
-            raise NotImplementedError
-
-        #If 'name' is not a field name we know of, we cannot do anything.
-        if not object.__getattribute__(self,'couldhavefield')(name):
-            raise NotImplementedError
-        #otherwise set the field to zero.
-        object.__getattribute__(self, 'addfield')(name, np.zeros(shape), False)
-    def copy_into(self, into):
-        """Helper function for copy(): deep copying."""
-        if not isinstance(into, AliasedArrayAttributes):
-            raise TypeError('copy_into() cannot copy into other types than \
-AliasedArrayAttributes or its subclasses')
-        AliasedAttributes.copy_into(self, into)
-        into._shape = self._shape
-        into._protnames = self._protnames[:]
-        into._normalnames = self._normalnames[:]
-    def shape(self):
-        """Return the (common) shape of the attributes"""
-        return self._shape
-    def __len__(self):
-        if self._shape is not None:
-            return reduce(lambda a, b:a*b, [x for x in self._shape if x > 0])
-        else:
-            return None
-    def _attr_validate(self, name, value):
-        """Validator function"""
-        if value is None:
-            if self._shape is None:
-                raise ValueError('None is not allowed for the first field in \
-DataSet!')
-            else:
-                # make it zero
-                value = np.zeros(self._shape)
-        value = np.array(value) # convert it to a numpy array
-        if not self._shape: # None or an empty tuple/list
-            self._shape = value.shape
-        if value.shape != self._shape:
-            raise ValueError('Invalid length for new field!')
-        return value
-    def __getitem__(self, key):
-        """Slicing. Forward arguments to the __getitem__ members of the aliased
-        attributes (of type numpy.ndarray)."""
-        #make a copy
-        obj = self.copy()
-        # set shape to None, to allow validation of different shaped fields.
-        obj._shape = None
-        for k in self.fields():
-            obj.addfield(k, self.getfield(k)[key], False)
-        obj._shape = obj.getfield(k).shape
-        return obj
-    def clear(self):
-        """Clear all attributes."""
-        AliasedAttributes.clear(self)
-        self._shape = None
         
 class AliasedVectorAttributes(AliasedArrayAttributes, ArithmeticBase):
     """A specialization of AliasedArrayAttributes with the constraint that the
@@ -406,7 +304,7 @@ AliasedVectorAttributes or its subclasses')
             obj.validate()
         return obj
 
-    def modulus(self, exponent = 0, errorrequested = False):
+    def momentum(self, exponent = 0, errorrequested = False):
         """Calculate moduli (integral from 0 to infinity of y times x^exponent)
         
         Inputs:
@@ -440,8 +338,6 @@ AliasedVectorAttributes or its subclasses')
             xmin=max(self._x.min(),dataset._x.min())
         if xmax is None:
             xmax=min(self._x.max(),dataset._x.max())
-        if xsep is None:
-            xsep=0.5*(xmin+xmax)
         if xmin>xmax:
             raise ValueError('Datasets do not overlap or xmin > xmax.')
         commonx=np.linspace(xmin,xmax,Npoints)
@@ -449,12 +345,30 @@ AliasedVectorAttributes or its subclasses')
         datasetint=dataset.interpolate(commonx)
         I1=ErrorValue(*(selfint.modulus(errorrequested=True)))
         I2=ErrorValue(*(datasetint.modulus(errorrequested=True)))
-        obj=self[self._x<=xsep]
+        obj=self.copy()
         print "I1:",I1
         print "I2:",I2
         print "Uniting factor:", (I1/I2)
         dataset=dataset*(I1/I2)
-        return obj.extend(dataset[dataset._x>xsep])
+        if xsep is not None:
+            return obj[obj._x<=xsep].extend(dataset[dataset._x>xsep])
+        else:
+            return obj.extend(dataset)
+    
+    @staticmethod
+    def average(*datasets):
+        if len(datasets)==1 and hasattr(datasets[0],'__getitem__'):
+            datasets=datasets[0]
+        res=datasets[0].copy()
+        res._y=np.zeros_like(res._y)
+        res._dy=np.zeros_like(res._y)
+        for ds in datasets:
+            w=1/ds._dy**2
+            res._y=res._y+ds._y*w
+            res._dy=res._dy+w
+        res._y=res._y/res._dy
+        res._dy=np.sqrt(1/res._dy)
+        return res
         
 
 
@@ -898,7 +812,7 @@ keyword argument \'%s\''%n)
                  transform = plt.gca().transAxes)
                  
     def fit(self, function, parinit = None, funcinfo = {}, doplot = True, 
-            ext_output = False):
+            ext_output = False, **kwargs):
         """Perform a least-squares fit to the dataset.
         
         Inputs:
@@ -918,7 +832,7 @@ keyword argument \'%s\''%n)
                 after updating with function.funcinfo if function was an
                 instance of FitFunction.
             ext_output: if extended output is needed (see outputs)
-            
+            All other keyword arguments are passed to scipy.optimize.leastsq()
         Outputs: p, pstd, [infodict]
             p: list of fitted parameters
             pstd: list of errors of fitted parameters
@@ -974,7 +888,8 @@ keyword argument \'%s\''%n)
         p, cov_x, infodict, mesg, ier = scipy.optimize.leastsq(func,
                                                                params_initial, 
                                                                args = (x, y, w), 
-                                                               full_output = 1)
+                                                               full_output = 1,
+                                                               **kwargs)
 #        if ier > 4 or ier < 1:
 #            raise DataSetError(mesg)
         chisquare = (infodict['fvec']**2).sum()
@@ -996,11 +911,15 @@ keyword argument \'%s\''%n)
             infodict['mesg'] = mesg
             infodict['ier'] = ier
             infodict['weighting'] = weighting
-            infodict['cov_x'] = cov_x
+            if cov_x is None:
+                infodict['cov_x']=np.zeros((len(p),len(p)))*np.nan
+            else:
+                infodict['cov_x'] = np.array(cov_x)*float(chisquare)/float(degrees_of_freedom)
             infodict['R2'] = r2
             return p, pstd, infodict
         else:            
             return p, pstd
+            
         
 class SASCurve(DataSet):
     def __init__(self, *args, **kwargs):
@@ -1024,4 +943,5 @@ def errtrapz(x, yerr):
     return 0.5*np.sqrt((x[1]-x[0])**2*yerr[0]**2+
                         np.sum((x[2:]-x[:-2])**2*yerr[1:-1]**2)+
                         (x[-1]-x[-2])**2*yerr[-1]**2)
-        
+
+
