@@ -1,4 +1,4 @@
-﻿# -*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 """
 Fit functions and transformations.
 
@@ -28,7 +28,7 @@ Conventions: names for subclasses of FitFunction should start with 'FF'. Names
 """
 
 import numpy as np
-from scipy.special import j1, gamma, gammaincc,gammainc
+from scipy.special import *
 import cfunctions
 
 class FitFunction(object):
@@ -204,7 +204,23 @@ class FFLinear(FitFunction):
         FitFunction.__init__(self)
     def __call__(self,x,a,b):
         return a*x+b
- 
+
+class FFCorrelatedRod(FitFunction):
+    name="Rod with correlation"
+    argument_info=[('A','scaling'),('L','rod length'),('Rc','Rc parameter'),('sigma','sigma parameter'),('B','coupling strength')]
+    formula="y(q) = A* P(q,L)/(1+B*sin(q*Rc)/(q*Rc)*exp(-q^2*sigma^2));\nP(q) = L^2*(2/(q*L)*Si(q*L)-sin(q*L/2)/(q*L/2))"
+    def __call__(self,q,A,L,Rc,sigma,B):
+        P=L*L*(2/(q*L)*sici(q*L)[0]-sinc(q*L/2.))
+        return A*P/(1+B*sinc(q*Rc)*exp(-q*q*sigma*sigma)*P)
+
+class FFCorrelatedPowerlaw(FitFunction):
+    name="Power-law with correlation"
+    argument_info=[('A','scaling'),('alpha','exponent'),('Rc','Rc parameter'),('sigma','sigma parameter'),('B','Coupling strength')]
+    formula="y(q) = A*q^-alpha/(1+B*sin(q*Rc)/(q*Rc)*exp(-q^2*sigma^2)*q^-alpha)"
+    def __call__(self,q,A,alpha,Rc,sigma,B):
+        P=np.power(q,alpha)
+        return A*P/(1+B*sinc(q*Rc)*exp(-q*q*sigma*sigma)*P)
+        
 class FFHorkay(FitFunction):
     name="Horkay et al. J. Chem. Phys. 125 234904 (9)"
     argument_info=[('F','factor'),
@@ -761,7 +777,7 @@ class FFLorentzian(FitFunction):
     def __init__(self):
         FitFunction.__init__(self)
     def __call__(self,x,A,x0,sigma,B):
-        return A/(1+((x-x0)/sigma)^2)+B
+        return A/(1+((x-x0)/sigma)**2)+B
 
 def Fsphere(q,R):
     return 4*np.pi/q**3*(np.sin(q*R)-q*R*np.cos(q*R))
