@@ -491,7 +491,15 @@ class FittingTool(Tk.Toplevel):
                 for n,v,e in zip(self.winfo_toplevel().fs.getfunction().argument_info,p,pstd):
                     self.winfo_toplevel().log("%s (%s)\t%g\t%g\n"%(n[0],n[1],v,e))
                 self.winfo_toplevel().log("\n")
-                self.winfo_toplevel().log("Message: (%d) %s\n"%(infodict['ier'],infodict['mesg']))
+                self.winfo_toplevel().log("Covariance matrix:\n")
+                cov_x=infodict['cov_x']
+                self.winfo_toplevel().log(str(cov_x))
+                self.winfo_toplevel().log('\n')
+                self.winfo_toplevel().log('Pearson\'s coefficients (r):\n')
+                sigma=np.sqrt(cov_x.diagonal())
+                r=cov_x/np.outer(sigma,sigma)
+                self.winfo_toplevel().log(str(r))
+                self.winfo_toplevel().log("\n\nMessage: (%d) %s\n"%(infodict['ier'],infodict['mesg']))
                 self.winfo_toplevel().log("\n")
                 self.winfo_toplevel().fs.setargs(p,pstd);
                 self.winfo_toplevel().clearfigure()
@@ -513,7 +521,7 @@ class FittingTool(Tk.Toplevel):
             self.frame.rowconfigure(0,weight=1)
             self.text=Tk.ScrolledText(self.frame,height=90,width=1)
             self.text.grid(sticky='NSEW')
-            self.text.text['state']='disabled' #prevent user editing
+            #self.text.text['state']='disabled' #prevent user editing
             self.text.text['width']=30
         def write(self,string):
             self.text.text['state']='normal'
